package mschmidt.komo.datatools;
/*
 * Instances of this object class are used to perform 
 * SQL-based utility operations (Reindex and Vacuum).
 *  These tasks are performed in a helper thread.
 */

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationActionBarAdvisor;
import mschmidt.komo.DbInfoDialog;
import mschmidt.komo.KomoPlugin;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.progress.IProgressConstants;

public class UtilitySqlJob extends Job {

	// Initialize variables used by jobs.
	private String thisJob;

	private String thisImage = new String();

	private String sqlString = new String();

	/**
	 * The constructor to create the job object.
	 * 
	 * @param jobName -
	 *            the name of the job, used in naming the progress monitor and
	 *            in dialog titles
	 * @param imageString -
	 *            the ApplicationImages variable associated with this job - may
	 *            be null
	 * @param stmt -
	 *            the SQL statement that the job will execute
	 */
	public UtilitySqlJob(final String jobName, final String imageString,
			final String stmt) {
		super(jobName);
		thisJob = jobName;
		thisImage = imageString;
		sqlString = stmt;
	}

	/**
	 * The job runnable.
	 * @param monitor - the job ProgressMonitor
	 * @return run - the job Istatus
	 */
	@Override
	protected final IStatus run(final IProgressMonitor monitor) {
		// Instantiate a string buffer for collecting information.
		StringBuffer sbInfo = new StringBuffer();
		// Initialize a boolean variable for keeping track of job status.
		boolean ranOK = true;
		// instantiate the SQL statement.
		Statement sqlStmt = null;
		
		try {
			int monitorWorked = 10;
			if (thisImage != null && thisImage.length() > 0) {
				setProperty(IProgressConstants.ICON_PROPERTY, KomoPlugin
						.getImageDescriptor(thisImage));
			}
			monitor.beginTask("Database " + thisJob, IProgressMonitor.UNKNOWN);
			sbInfo.append("*** Execute Statement; " + sqlString
					+ "\n*** Begin Processing Log\n\n");
			Application.dbConn.dbLock.acquire();
			Connection thisConnection = Application.dbConn.getConnection();
			if (thisConnection == null) {
				sbInfo.append("Database connection could not be established.\n\n*** " 
						+ thisJob + " operation failed.\n");
				ranOK = false;
				return Status.CANCEL_STATUS;
			}
			monitor.worked(monitorWorked);
			sqlStmt = thisConnection.createStatement();
			sqlStmt.execute(sqlString);
			monitor.worked(10);
			SQLWarning warnStmt = sqlStmt.getWarnings();
			while (warnStmt != null) {
				sbInfo.append(warnStmt + "\n");
				warnStmt = warnStmt.getNextWarning();
				monitor.worked(monitorWorked);
			}
			sbInfo.append("\n*** End Processing Log.\n");

		} catch (Exception e) {
			sbInfo.append(e.getMessage() + "\n\n*** " + thisJob
					+ " operation failed.\n");
			ranOK = false;
			e.printStackTrace();

		} finally {
			try {
				if (sqlStmt != null) {
					sqlStmt.close();
				}
			} catch (SQLException e) {
				sbInfo.append(e.getMessage() + "\n");
				e.printStackTrace();
			}
			monitor.done();
			Application.dbConn.closeConnection();
			Application.dbConn.dbLock.release();
			showResults(sbInfo.toString(), ranOK);
		}
		return Status.OK_STATUS;
	}

	/*
	 * Output results upon completion of task.
	 */
	protected final void showResults(final String message, final boolean ranOK) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if (!ranOK) {
					String dbMsg = Application.dbConn.getMessage();
					ApplicationActionBarAdvisor.postMessage(dbMsg, true);
				}
				DbInfoDialog dbDialog = new DbInfoDialog(Display.getDefault()
						.getActiveShell(), message);
				dbDialog.create();
				dbDialog.getShell().setText(thisJob + " Results");
				if (thisImage != null && thisImage.length() > 0) {
					dbDialog.getShell().setImage(
							KomoPlugin.getImageDescriptor(thisImage)
									.createImage());
					dbDialog.open();
				}
			}
		});
	}

}
